# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/tools/targets_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_targets(object):
    def setupUi(self, targets):
        targets.setObjectName("targets")
        targets.resize(322, 216)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("manuskript/ui/tools/../../../../../../../../../../home/icons/Manuskript/icon-64px.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        targets.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(targets)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.line = QtWidgets.QFrame(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.draft_progress_bar = QtWidgets.QProgressBar(targets)
        self.draft_progress_bar.setProperty("value", 0)
        self.draft_progress_bar.setObjectName("draft_progress_bar")
        self.verticalLayout_2.addWidget(self.draft_progress_bar)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.draft_wc_label = QtWidgets.QLabel(targets)
        self.draft_wc_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.draft_wc_label.setObjectName("draft_wc_label")
        self.horizontalLayout_4.addWidget(self.draft_wc_label)
        self.draft_of_label = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.draft_of_label.sizePolicy().hasHeightForWidth())
        self.draft_of_label.setSizePolicy(sizePolicy)
        self.draft_of_label.setObjectName("draft_of_label")
        self.horizontalLayout_4.addWidget(self.draft_of_label)
        self.draft_goal_label = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.draft_goal_label.sizePolicy().hasHeightForWidth())
        self.draft_goal_label.setSizePolicy(sizePolicy)
        self.draft_goal_label.setObjectName("draft_goal_label")
        self.horizontalLayout_4.addWidget(self.draft_goal_label)
        self.label_8 = QtWidgets.QLabel(targets)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_4.addWidget(self.label_8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(targets)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.line_2 = QtWidgets.QFrame(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_2.addWidget(self.line_2)
        self.session_reset = QtWidgets.QPushButton(targets)
        self.session_reset.setObjectName("session_reset")
        self.horizontalLayout_2.addWidget(self.session_reset)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.session_progress_bar = QtWidgets.QProgressBar(targets)
        self.session_progress_bar.setProperty("value", 0)
        self.session_progress_bar.setObjectName("session_progress_bar")
        self.verticalLayout_2.addWidget(self.session_progress_bar)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.session_wc_label = QtWidgets.QLabel(targets)
        self.session_wc_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.session_wc_label.setObjectName("session_wc_label")
        self.horizontalLayout_3.addWidget(self.session_wc_label)
        self.label_3 = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.session_target = QtWidgets.QDoubleSpinBox(targets)
        self.session_target.setDecimals(0)
        self.session_target.setMaximum(99999999999999.0)
        self.session_target.setSingleStep(50.0)
        self.session_target.setProperty("value", 500.0)
        self.session_target.setObjectName("session_target")
        self.horizontalLayout_3.addWidget(self.session_target)
        self.label_5 = QtWidgets.QLabel(targets)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_3.addWidget(self.label_5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.retranslateUi(targets)
        QtCore.QMetaObject.connectSlotsByName(targets)

    def retranslateUi(self, targets):
        _translate = QtCore.QCoreApplication.translate
        targets.setWindowTitle(_translate("targets", "Project Targets"))
        self.label.setText(_translate("targets", "Draft Target"))
        self.draft_wc_label.setText(_translate("targets", "0"))
        self.draft_of_label.setText(_translate("targets", "of"))
        self.draft_goal_label.setText(_translate("targets", "10,000"))
        self.label_8.setText(_translate("targets", "Words"))
        self.label_2.setText(_translate("targets", "Session Target"))
        self.session_reset.setText(_translate("targets", "Reset"))
        self.session_wc_label.setText(_translate("targets", "0"))
        self.label_3.setText(_translate("targets", "of"))
        self.label_5.setText(_translate("targets", "words"))
